#
# maint-dev-cost-ratio.R, 22 Oct 12
#
# R code for book "Empirical Software Engineering using R"
# Derek M. Jones, http://shape-of-code.coding-guidelines.com
#
# Data from paper "An Investigation of the Factors Affecting the Lifecycle Costs of COTS-Based Systems"
# by Laurence Michael Dunn


library(fitdistrplus)

dme=read.csv("/usr1/rbook/examples/economics/maint-invest//dev-maint-effort.csv")

par(las=1)
par(bty="l")
par(col="black")

par(mfcol=c(1,1))

y=sort(dme$dev.effort/dme$maint.effort, decreasing=TRUE)
#descdist(y, boot=100)

t=fitdist(y/(max(y)+1), distr="beta", start=list(shape1=0.2, shape2=10), method="mle")
#summary(t)
tb=dbeta(seq(1/length(y), 1, by=1/length(y)), shape1=t$estimate[1], shape2=t$estimate[2])

# fitdist requires values in [0, 1] while dbeta can return values
# outside that range, so some scaling is required.  We could do
# lots of work and get it correct, choose the first element to align
# or do it by eye (10 looks good).
#norm=dbeta(1/length(y), shape1=t$estimate[1], shape2=t$estimate[2])
norm=10

plot(tb*max(y)/norm, ylim=c(0, 12),
       xlab="Sorted list of systems", ylab="Development/Maintenance")
par(new=TRUE)
plot(y, ylim=c(0, 12), col="red",
       xlab="", ylab="")

